#
#	tonclib.mak
#
# Making my gba utilities

# --- Project details ---

PROJ    := libtonc

TONCLIB := $(PROJ).a

# --- Source files ---
ICSRC   := $(wildcard *.iwram.c)
RCSRC   := $(filter-out $(ICSRC), $(wildcard *.c))
CSRC    := $(RCSRC) $(ICSRC)
# I want to see the asm code of the c-files too, but not of the
# luts, so filter those out first. And also filter out the generated
# s files (from .c -> .s) from the true assembly files
SSRC    := $(filter-out $(CSRC:.c=.s), $(wildcard *.s))

# Generate asm for C files (yes/no: `1'/`0')
ASMOUT  := 1

# --- commands ---
CROSS   := arm-eabi-
AR      := $(CROSS)ar
AS      := $(CROSS)gcc
CC      := $(CROSS)gcc

ARCH    := -mthumb-interwork -mthumb
RARCH   := -mthumb-interwork -mthumb
IARCH   := -mthumb-interwork -marm

CBASE   := $(INCDIR) -O2 -Wall -fno-strict-aliasing
RCFLAGS := $(CBASE) $(RARCH)
ICFLAGS := $(CBASE) $(IARCH) -mlong-calls
CFLAGS  := $(RCFLAGS)

ASFLAGS := -x assembler-with-cpp  -c -mthumb-interwork

# --- objects files ---

LUTOBJ  := $(LUTSRC:.c=.o)
RCOBJ   := $(RCSRC:.c=.o)
ICOBJ   := $(ICSRC:.c=.o)
COBJ    := $(RCOBJ) $(ICOBJ)
SOBJ    := $(SSRC:.s=.o)

OBJ     := $(COBJ) $(SOBJ)

# === main targets ===

# default: just compile the files
build: depends $(OBJ)

# buildlib: create a library
buildlib : depends $(TONCLIB)

$(TONCLIB) : $(OBJ)
	$(AR) rcs $@ $^

# === code generation ===

# --- ROM compilation ---
# --- *.c -> *.o --------
$(RCOBJ) : %.o : %.c
	$(CC) $(RCFLAGS) -c $< -o $@
ifeq ($(ASMOUT), 1)
	$(CC) $(RCFLAGS) -S $<
endif

# --- IWRAM compilation ----
# *.iwram.c -> *.iwram.o ---
$(ICOBJ) : %.iwram.o : %.iwram.c
	$(CC) $(ICFLAGS) -c $< -o $@
ifeq ($(ASMOUT), 1)
	$(CC) $(ICFLAGS) -S $<
endif

# --- LUT compilation ---
$(LUTOBJ): %.o : %.c
	$(CC) $(CFLAGS) -c $< -o $@


# --- Assembling ---
# --- *.s -> *.o ---
$(SOBJ) : %.o : %.s
	$(AS) $(ASFLAGS) -c $< -o $@

# --- Header dependencies ---
# --- *.c -> .d -------------
depends:
	$(CC) -MM $(CFLAGS) -c $(ICSRC) $(RCSRC) > $(PROJ).d

-include $(PROJ).d

.PHONY : clean
clean: 
	rm -fv $(OBJ)
	rm -fv $(CSRC:.c=.s)
